
% For a given distribution, computes welfare gains from changing trade
% costs, along with predictions from ACR and ACDR models.

intervalSize = 0.02;
stepsBelow = ceil((1 - exp(-0.2)) / intervalSize);
stepsAbove = ceil(-(1 - exp(0.2)) / intervalSize);

lb = 1 - stepsBelow * intervalSize;
ub = 1 + stepsAbove * intervalSize;
n = stepsBelow + stepsAbove + 1;

taus = linspace(lb, ub, n) .* tauInit;
expenditures = zeros(1, n);
equilibria = cell(1, n);
lambdajjs = zeros(1, n);
utilities = zeros(1, n);
rhos_d = zeros(1, n);
rhos_x = zeros(1, n);
Ns = zeros(1, n);

% For each tau, solve for equilibrium and extract relevant statistics.
for i = 1:n
    tau = taus(i)
    [equil, stats] = solveEquilibrium(tau, p_star);
    lambdajjs(i) = 1 - stats.exportShare;
    p_star = equil.p_star;
    rhos_d(i) = stats.rho_d;
    rhos_x(i) = stats.rho_x;
    equilibria{i} = equil;
    utilities(i) = calculateUtility(equil);
    Ns(i) = equil.N;
end

baselineIndex = stepsBelow + 1; % Index of calibrated tau (tauInit)
baselineUtility = utilities(baselineIndex);

welfareGainsExact = zeros(1, n);

% Compute exact welfare gains using expenditure and utility.
for i = 1:n
    if (useAnalyticExpenditure)
        welfareGainsExact(i) = analyticExpenditure(equilibria{baselineIndex}, utilities(i)) /...
            analyticExpenditure(equilibria{i}, utilities(i));
    else
        welfareGainsExact(i) = numericalExpenditure(equilibria{baselineIndex}, utilities(i), taus(baselineIndex)) /...
            numericalExpenditure(equilibria{i}, utilities(i), taus(i));
    end
end

normTradeFlows = (1-lambdajjs)./lambdajjs;
elasticities = diff(log(normTradeFlows)) ./ diff(log(taus));

% Compute predicted changes in welfare using ACR model.
dWelfareACR = (lambdajjs(1:(n-1))./lambdajjs(2:n)) .^ (-1./elasticities);

% Compute predicted changes in welfare using ACDR model.
eta = rhos_d(1:(n-1)) ./ (1-elasticities);
dWelfareACDR = (lambdajjs(1:(n-1))./lambdajjs(2:n)) .^ (-(1-eta)./elasticities);

welfareGainsACR = ones(1, n);
welfareGainsACDR = ones(1, n);

% Now multiply successive changes in welfare (ACR, ACDR) to get
% exact and predicted welfare at each tau.
for i = 1:n    
    % For tau < tauInit, we must invert the result (since we're moving
    % backward from the baseline).
    if i < baselineIndex
        welfareGainsACR(i) = prod(dWelfareACR(1, i:(baselineIndex-1))) ^ (-1);
        welfareGainsACDR(i) = prod(dWelfareACDR(1, i:(baselineIndex-1))) ^ (-1);
    elseif i > baselineIndex
        welfareGainsACR(i) = prod(dWelfareACR(1, baselineIndex:(i-1)));
        welfareGainsACDR(i) = prod(dWelfareACDR(1, baselineIndex:(i-1)));
    end
end

dlogTaus = log(taus./tauInit);
dlogLambdas = log(lambdajjs./lambdajjs(baselineIndex));
diffsACR = log(welfareGainsACR./welfareGainsExact);
diffsACDR = log(welfareGainsACDR./welfareGainsExact);

% Make plots

figure(nFig)
hold on;
X = ['Welfare Gains Relative to Baseline ( \tau = ', num2str(tauInit), '), ', distribution];
%title(X, 'FontSize', titleSize);
xlabel('dln( \tau)', 'FontSize', labelSize)
ylabel('Percentage Gains', 'FontSize', labelSize)
plot(dlogTaus, (welfareGainsExact-1)*100, '-k', dlogTaus, (welfareGainsACR-1)*100, '--k', ...
    dlogTaus, (welfareGainsACDR-1)*100, ':k', 'LineWidth', 1);
legend('Exact', 'ACR', 'ACDR');
hold off;

figure(nFig+1)
hold on;
title(['Log differences between Exact and AC(D)R calculations, ', distribution], 'FontSize', titleSize);
xlabel('dln(\tau)', 'FontSize', labelSize)
ylabel('Relative difference (dlnW_{AC(D)R} - dlnW_{Exact})', 'FontSize', labelSize)
plot(dlogTaus, diffsACR, '--k', dlogTaus, diffsACDR, ':k', 'LineWidth', 1);
legend('ACR', 'ACDR');
hold off;

figure(nFig+2)
hold on;
title(['Log differences between Exact and AC(D)R calculations, ', distribution], 'FontSize', titleSize);
xlabel('dln(\lambda_{jj})', 'FontSize', labelSize)
ylabel('Relative difference (dlnW_{AC(D)R} - dlnW_{Exact})', 'FontSize', labelSize)
plot(dlogLambdas, diffsACR, '--k', dlogLambdas, diffsACDR, ':k', 'LineWidth', 1);
legend('ACR', 'ACDR');
hold off;
