function [equilibrium, statistics] = solveEquilibrium(tau, p_star_init)
% Solves for two-country symmetric equilibrium for given tau (and other
% external parameters).

global z_lb f_d f_x f_e L g q tol adj distribution phimin

options = optimoptions('fsolve', 'Algorithm', 'trust-region-dogleg', 'TolFun', 1e-8, 'Display', 'off');

err = 1;
p_star = p_star_init;

% Guess p_star, then calculate z_d, z_x, N. Use (4) to check consistency,
% then adjust p_star accordingly.
while (abs(err) > tol)
    
    disp(['Trying p* = ', num2str(p_star), ' ...']);
    
    [p_d, p_x] = computePrices(tau, p_star);
    
    prof_d = @(z) (p_d(z) - 1./z) .* q(p_d(z), p_star); % Domestic profits by firm
    prof_x = @(z) (p_x(z) - tau./z) .* q(p_x(z), p_star); % Export profits by firm
    
    h1 = @(z) prof_d(z) - f_d; % Equation (1)
    h2 = @(z) prof_x(z) - f_x; % Equation (2)

    z_d = fsolve(h1, z_lb, options); % Domestic productivity cutoff
    z_x = fsolve(h2, z_lb, options); % Export productivity cutoff  
    
    fun1 = @(z) (f_d + q(p_d(z), p_star)./z) .* g(z);
    fun2 = @(z) (f_x + q(p_x(z), p_star)./(z/tau)) .* g(z);

    totalDomesticProfits = integral(fun1, z_d, Inf);
    totalExportProfits = integral(fun2, z_x, Inf);

    N = L / (f_e + totalDomesticProfits + totalExportProfits); % Equation (3)
    
    disp(['z_d = ', num2str(z_d), ',  z_x = ', num2str(z_x), ',  N = ', num2str(N)]);

    x_d = @(z) p_d(z) .* q(p_d(z), p_star); % Domestic sales by firm
    x_x = @(z) p_x(z) .* q(p_x(z), p_star); % Export sales by firm
    
    fun3 = @(z) x_d(z) .* g(z);
    fun4 = @(z) x_x(z) .* g(z);
    
    totalDomesticSales = integral(fun3, z_d, Inf);
    totalExportSales = integral(fun4, z_x, Inf);
    
    errOld = err;
    err = N * (totalDomesticSales + totalExportSales) - 1; % Equation (4)
    disp(['[err = ', num2str(err), ']']);
    disp(' ');
    
    if (abs(err) >= 0.99*abs(errOld))
        warning('Equilibrium solver not converging with sufficient precision. Leaving loop now.');
        break;
    end
    
    p_star = max(0.1, p_star - adj * err);
    
end

if strcmp(distribution, 'Pareto') || strcmp(distribution, 'Truncated Pareto')
    if z_d < phimin
        msg = 'z* is smaller than phimin.';
        error(msg)    
    end
end

% Put values for solved equilibrium in structure
f1 = 'z_d';     v1 = z_d;
f2 = 'z_x';     v2 = z_x;
f3 = 'N';       v3 = N;
f4 = 'p_star';  v4 = p_star;
f5 = 'p_d';     v5 = p_d;
f6 = 'p_x';     v6 = p_x;

equilibrium = struct(f1, v1, f2, v2, f3, v3, f4, v4, f5, v5, f6, v6);

% Compute statistics
numExporters = integral(g, z_x, Inf);
numTotal = integral(g, z_d, Inf);
fracExporters = numExporters / numTotal;

exportShare = totalExportSales / (totalDomesticSales + totalExportSales);

normTradeFlows = totalExportSales / totalDomesticSales;

domesticProfitRatio = totalDomesticProfits / totalDomesticSales;
exportProfitRatio = totalExportProfits / totalExportSales;

rho_d = calculateRho(z_d, p_star, p_d, 1); % Domestic trade cost = 1
rho_x = calculateRho(z_x, p_star, p_x, tau);

f1 = 'fracExporters';   v1 = fracExporters;
f2 = 'exportShare';     v2 = exportShare;
f3 = 'normTradeFlows';  v3 = normTradeFlows;
f4 = 'domesticProfitRatio'; v4 = domesticProfitRatio;
f5 = 'exportProfitRatio'; v5 = exportProfitRatio;
f6 = 'rho_d';           v6 = rho_d;
f7 = 'rho_x';           v7 = rho_x;

statistics = struct(f1, v1, f2, v2, f3, v3, f4, v4, f5, v5, f6, v6, f7, v7);

end

